package cn.estsh.i3plus.ext.mes.pcn.pojo.model.bu3;

import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.MesEsopExt;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiParam;
import lombok.Data;

import java.io.Serializable;
import java.util.List;

/**
 * @author wangjie
 * @version 1.0
 * @date 2021/1/15 15:32
 **/
@Data
@ApiModel("SOP数据")
public class SxEsopCenterDataModel implements Serializable {

    private static final long serialVersionUID = -8081385641232929605L;

    @ApiParam("页面行为:查询=1,自动切替=2,手动切替=3")
    private Integer actionType;

    @ApiParam("手动切替方式:上=1,下=2")
    private Integer handFlag;

    @ApiParam("区域")
    private String areaCode;

    @ApiParam("生产线")
    private String workCenterCode;

    @ApiParam("工位")
    private String workCellCode;

    @ApiParam("工序代码")
    private String processCode;

    @ApiParam("工序名称")
    private String processName;

    @ApiParam("工单号")
    private String workOrderNo;

    @ApiParam("物料号")
    private String partNo;

    @ApiParam("物料名称")
    private String partNameRdd;

    @ApiParam("SOP集合")
    private List<MesEsopExt> esopExtList;

    @ApiParam("工厂")
    private String organizeCode;

    @ApiParam("信息")
    private String msg;

    @ApiParam("反馈信息")
    private String feedBackMsg;

    public SxEsopCenterDataModel() {}

    public SxEsopCenterDataModel processCode(String processCode) {
        this.processCode = processCode;
        return this;
    }

    public SxEsopCenterDataModel processName(String processName) {
        this.processName = processName;
        return this;
    }

    public SxEsopCenterDataModel workOrderNo(String workOrderNo) {
        this.workOrderNo = workOrderNo;
        return this;
    }

    public SxEsopCenterDataModel partNo(String partNo) {
        this.partNo = partNo;
        return this;
    }

    public SxEsopCenterDataModel partNameRdd(String partNameRdd) {
        this.partNameRdd = partNameRdd;
        return this;
    }

    public SxEsopCenterDataModel msg(String msg) {
        this.msg = msg;
        return this;
    }

    public SxEsopCenterDataModel feedBackMsg(String feedBackMsg) {
        this.feedBackMsg = feedBackMsg;
        return this;
    }
}
